module Ewigkeks
  class CookiesController < ApplicationController
    def png
      if cookies[Ewigkeks.png_cookie_name].present?
        response.headers['Content-Type']  = 'image/png'
        response.headers['Last-Modified'] = 'Wed, 30 Jun 2010 21:36:48 GMT'
        response.headers['Expires']       = 'Tue, 31 Dec 2030 23:30:45 GMT'
        response.headers['Cache-Control'] = 'private, max-age=630720000'

        render text: png_blob, content_type: 'image/png'
      else
        head status: :not_modified
      end
    end

    def cache
      if cookies[Ewigkeks.cache_cookie_name].present?
        response.headers['Content-Type']  = 'text/html'
        response.headers['Last-Modified'] = 'Wed, 30 Jun 2010 21:36:48 GMT'
        response.headers['Expires']       = 'Tue, 31 Dec 2030 23:30:45 GMT'
        response.headers['Cache-Control'] = 'private, max-age=630720000'

        render text: cookies[Ewigkeks.cache_cookie_name]
      else
        head status: :not_modified
      end
    end

    def etag
      if cookies[Ewigkeks.etag_cookie_name].present?
        response.headers['Etag'] = cookies[Ewigkeks.etag_cookie_name]
        render text: cookies[Ewigkeks.etag_cookie_name]
      else
        render text: request.headers.fetch('If-None-Match', ''), status: :not_modified
      end
    end

    private

    def png_blob
      value = cookies[Ewigkeks.png_cookie_name]
      image = ChunkyPNG::Image.new(200, 1, ChunkyPNG::Color::BLACK)

      (0..value.length).step(3) do |index|
        image[(index/3).round, 0] = pixel_by_index(value, index)
      end

      image.to_blob(color_mode: ChunkyPNG::COLOR_TRUECOLOR, compression: Zlib::DEFAULT_COMPRESSION)
    end

    def pixel_by_index(value, index)
      red   = value[index]     ? value[index].ord     : 0
      green = value[index + 1] ? value[index + 1].ord : 0
      blue  = value[index + 2] ? value[index + 2].ord : 0

      ChunkyPNG::Color.rgb(red, green, blue)
    end
  end
end
